#!/usr/bin/perl -w

# The purpose of this file
# ------------------------
#
# Both the Perl parts and the JavaScript parts of CCamelOT need access to phonemes.
# In order to keep a single phoneme list in the system, this file writes both
# the Perl module and the JavaScript file that are needed.
#
# To change phonemes and features, follow these steps: 
# 1. Use Excel to update the file "Phoneme.xls". 
# 2. Save "Phoneme.xls" as a tab-delimited text file, and call it "Phoneme.txt". 
# 3. Use a text editor to replace all triple double quotes (""") with single double quotes ("). 
# 4. Run this script ("Phoneme_generator.cgi") - other directly, from the command line,
#    or with the conveniet "Phoneme_generator.bat"  
#

use strict;
use warnings;

use Fcntl qw(:flock);

open (INPUT,  "<" . "Phoneme.txt") or die "Can't read phonemes.txt $!";
open (OUTPM,  ">" . "Phoneme.pm")  or die "Can't write to Phoneme.pm $!";
open (OUTJS,  ">" . "Phoneme.js")  or die "Can't write to Phoneme.js $!";

flock(INPUT, LOCK_EX) or die "$!";
flock(OUTPM, LOCK_EX) or die "$!";
flock(OUTJS, LOCK_EX) or die "$!";


print OUTPM <<'EOF';
#
#
#    DO NOT EDIT THIS FILE
#
#    This file is generated by "Phoneme_generator.cgi",
#    do your editing there.
#
#
package Phoneme;
use strict;
our (@ISA, @EXPORT);

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(seg setAlphabet changeFeature);


sub new {
	my $class = shift;
	my $self = {};
	bless($self,$class);

	$self->{symbol} = "" ;
	$self->{APA} = shift ;
	$self->{APAU} = shift ;
	$self->{IPA} = shift ;
	$self->{IPAU} = shift ;

	$self->{cons}   = shift ;
	$self->{sonor}  = shift ;
	$self->{place}  = shift ;
	$self->{ant}    = shift ;
	$self->{dist}   = shift ;
	$self->{lat}    = shift ;
	$self->{strid}  = shift ;
	$self->{son}    = shift ;
	$self->{cont}   = shift ;
	$self->{voice}  = shift ;
	$self->{cgl}    = shift ;
	$self->{sgl}    = shift ;
	$self->{high}   = shift ;
	$self->{back}   = shift ;
	$self->{low}    = shift ;
	$self->{round}  = shift ;
	$self->{ATR}    = shift ;
	$self->{nasal}  = shift ;

	return $self;
}

### read and write method
sub symbol { 
	my $self = shift; 
	if (@_) { $self->{symbol} = shift; };
	return $self->{symbol};
};




### read-only methods
sub IPA    { my $self = shift; return $self->{IPA}    };
sub IPAU   { my $self = shift; return $self->{IPAU}   };
sub APA    { my $self = shift; return $self->{APA}    };
sub APAU   { my $self = shift; return $self->{APAU}   };

sub cons   { my $self = shift; return $self->{cons}   };
sub sonor  { my $self = shift; return $self->{sonor}  };
sub place  { my $self = shift; return $self->{place}  };
sub ant    { my $self = shift; return $self->{ant}    };
sub dist   { my $self = shift; return $self->{dist}   };
sub lat    { my $self = shift; return $self->{lat}    };
sub strid  { my $self = shift; return $self->{strid}  };
sub son    { my $self = shift; return $self->{son}    };
sub cont   { my $self = shift; return $self->{cont}   };
sub voice  { my $self = shift; return $self->{voice}  };
sub cgl    { my $self = shift; return $self->{cgl}    };
sub sgl    { my $self = shift; return $self->{sgl}    };
sub high   { my $self = shift; return $self->{high}   };
sub back   { my $self = shift; return $self->{back}   };
sub low    { my $self = shift; return $self->{low}    };
sub round  { my $self = shift; return $self->{round}  };
sub ATR    { my $self = shift; return $self->{ATR}    };
sub nasal  { my $self = shift; return $self->{nasal}  };






# define segments

my %seg ;

EOF


print OUTJS <<'EOF';

//
//    DO NOT EDIT THIS FILE
//
//    This file is generated by "Phoneme_generator.cgi",
//    do your editing there.
//

///////// definition of phonemes

function phoneme(APA, APAU, IPA, IPAU, consonantal, sonor, place, ant, dist, lat, strid, son, cont, voice, cg, sg, high, back, low, round, ATR, nasal){
	this.symbol = "";
	this.APA    = APA;
	this.APAU   = APAU;
	this.IPA    = IPA;
	this.IPAU   = IPAU;

	this.cons   = consonantal
	this.sonor	= sonor;
	this.place	= place;
	this.ant	= ant;
	this.dist	= dist;
	this.lat	= lat;
	this.strid	= strid;
	this.son	= son;
	this.cont	= cont;
	this.voice	= voice;
	this.cg	= cg;
	this.sg	= sg;
	this.nasal	= nasal;

	this.high   = high;
	this.low    = low;
	this.back   = back;
	this.round  = round;
	this.ATR    = ATR;
        
}

var seg = []

EOF





while (<INPUT>) {
    unless (/^$/ or /^\t/) {
        chomp;
        my @currLine = split(/\t/);
        my $currPhoneme = shift(@currLine);
    
        print OUTPM '$seg{' , $currPhoneme , '} = Phoneme->new(';
        print OUTPM join(",",@currLine);
        print OUTPM ");\n";
        
        print OUTJS 'seg["' , $currPhoneme , '"] = new phoneme(';
        print OUTJS join(",",@currLine) ;
        print OUTJS ");\n";
    }
}



print OUTPM <<'EOF';


sub seg {
	return $seg{(shift)};
}

sub setAlphabet {

	my $alphabet = shift || "APAU";

	for my $currSeg (keys %seg) {
		$seg{$currSeg}->symbol($seg{$currSeg}->APAU()) if ($alphabet eq "APAU");	
		$seg{$currSeg}->symbol($seg{$currSeg}->APA() ) if ($alphabet eq "APA" );	
		$seg{$currSeg}->symbol($seg{$currSeg}->IPAU()) if ($alphabet eq "IPAU");	
		$seg{$currSeg}->symbol($seg{$currSeg}->IPA() ) if ($alphabet eq "IPA" );	
	}
	
	return $alphabet;
}


sub changeFeature {
	my $phonemeToReplace = shift;
	my $featureToChange = shift;

	my %newSeg = %{$seg{$phonemeToReplace}};
	$newSeg{$featureToChange} = ($newSeg{$featureToChange} eq "+") ? "-" : "+";
	
	my $foundNewSeg = "";
	PHON: for my $phon (keys %seg){
		  for my $feature (sort keys %{$seg{$phon}}) {
			    if ($feature !~ /PA/ and $feature ne "sonor" and $feature ne "symbol") {
					next PHON if ($newSeg{$feature} ne ${$seg{$phon}}{$feature});
			    }
		  }
		  # this line will be reached only for the segment that has the right
		  # value for all features, excluding the features "sonor", "symbol" and the alphabet features
		  # because of the "next PHON" command earlier
		  $foundNewSeg = $phon;
	}
	return $foundNewSeg;
}



1

EOF




print OUTJS <<'EOF';

function setAlphabet(alphabet,font) {

	if (alphabet != "IPA") alphabet = "APA";
	if (font != "") font = "U";
	if (font=="U") alphabet += font;

	for (var i in seg) {
		if (alphabet == "APA" ) seg[i].symbol = seg[i].APA;
		if (alphabet == "APAU") seg[i].symbol = seg[i].APAU;
		if (alphabet == "IPA" ) seg[i].symbol = seg[i].IPA;
		if (alphabet == "IPAU") seg[i].symbol = seg[i].IPAU;
	}

	d = new Date("January 01 2010");
	document.cookie = "alphabet=" + alphabet + ";expires=" + d.toGMTString();
}

EOF



close(INPUT) or die "$!";
close(OUTPM) or die "$!";
close(OUTJS) or die "$!";


